<?php

namespace wisdmlabs\cpb;

if (!class_exists('WdmProductBundleMain')) {
    /**
    * This class is responsible for :
    * loading CPB functionalities, activate CPB, add product-type CPB,
    * and set the class name for CPB.
    */
    class WdmProductBundleMain
    {
        /**
        * Get the license data, and then,
        * Adds following actions :
        * 1: Action for loading CPB Functionalities.
        * 2: Action to activate CPB.
        * 3: Action to add the product type of CPB.
        * 4: Action to set the product class name for CPB Product
        */
        public function __construct()
        {
            //Subscriptions
            $this->bundlePluginsLoaded();

            add_action('init', array($this, 'loadCPB'));
            add_action('wp', array($this, 'createGlobalProductObject'));
        }

        public function loadCPB()
        {
            global $cpbPluginData;
            $getDataFromDb = \Licensing\WdmLicense::checkLicenseAvailiblity($cpbPluginData['pluginSlug']);

            if ($getDataFromDb != 'available') {
                return;
            }
            
            $this->wdmCpbLoadTextDomain();
            $this->cpbMigration();
            $this ->product_type = 'wdm_bundle_product';
            add_action('admin_init', array($this, 'wdmActivate'));
            // add_action('wp_ajax_wdm_product_type_cpb', array($this, 'wdmProductTypeCpb'));
            // add_action('wp_ajax_nopriv_wdm_product_type_cpb', array($this, 'wdmProductTypeCpb'));
            add_filter('woocommerce_product_class', array($this,'fixProductTypeClassName'), 10, 4);

            //add_action('wp_head', array($this,'loadMediaManager'));
        }

        /**
        * Sets global variable of plugin data
        */


        public function cpbMigration()
        {
            //migration
            WdmCpbInstall::alterTable();
            WdmCpbInstall::changeCPBArray();
            WdmCpbInstall::updateUniqueID();
        }

        /**
        * Returns the absolute path of plugin directory.
        * @return string absolute path of plugin directory.
        */
        public function pluginPath()
        {
            return untrailingslashit(plugin_dir_path(__FILE__));
        }
        /**
        * Returns the absolute url of plugin directory.
        * @return string absolute url of plugin directory.
        */
        public function pluginUrl()
        {
            return untrailingslashit(plugins_url('/', __FILE__));
        }

        // fix bundle product type classname for PSR2 standards

        /**
        * Returns the product class name for CPB Product.
        * @param string $classname product class for different product type.
        * @return string $classname bundle product class
        */
        public function fixProductTypeClassName($classname)
        {
            if ($classname == 'WC_Product_Wdm_bundle_product') {
                return '\wisdmlabs\cpb\WCProductWdmBundleProduct';
            } else {
                return $classname;
            }
        }

        // public function loadMediaManager()
        // {
        // //wp_enqueue_media();
        // }

        /**
        * Return the CPB Plugin URL.
        * @return string CPB Plugin URL.
        */
        public function wooWdmBundlesPluginUrl()
        {
            return untrailingslashit(plugins_url('/', __FILE__));
        }

        /**
        * Load the Plugins files having the functionalities for admin and
        * front-end sides.
        */
        public function bundlePluginsLoaded()
        {
            global $woocommerce;

            //Add the file for woocommerce Compatibility for cpb plugin.
            if (version_compare($woocommerce->version, '2.0.22') > 0) {
                include('includes/woocommerce-21-functions.php');
            } else {
                include('includes/woocommerce-20-functions.php');
            }

            //File for CPB functions.
            require_once('includes/wdm-cpb-function.php');
            
            //File for MAin CPB Product
            require_once('includes/class-wc-product-wdm-product-bundle.php');

            require_once('includes/class-wdm-wc-product-item.php');
            require_once('includes/class-wdm-manage-prefill-data.php');
            require_once('includes/class-wdm-cpb-send-admin-notification.php');
            require_once('admin/class-wdm-cbp-layout-setting.php');

            // Admin functions and meta-boxes
            if (is_admin()) {
                // settings page related functions and filters
                require_once('includes/settings/class-cpb-settings-page.php');
                $this->settingPage = new WdmCpbSettingPage();

                require_once('admin/class-wdm-admin-asset.php');
                require_once('admin/class-wdm-product-bundle-type.php');
                $this->admin = new WdmProductBundleType();

                //This abstract class for privacy and GDPR functionality
                require_once('includes/privacy/class-abstract-cpb-privacy.php');

                //This file adds privacy and GDPR functionality
                require_once('includes/privacy/class-cpb-privacy.php');
            }

            // Cart & order -related bundle functions and filters
            require_once('includes/class-wdm-cpb-order-meta.php');
            require_once('includes/class-wdm-wc-product-cart.php');
            $this->cart = new WdmWcProductCart();

            // Front-end filters
            require_once('includes/simple_html_dom.php');
            require_once('public/class-wdm-frontend-scripts.php');

            // Gift Message
            require_once('public/class-wdm-gift-message.php');
            require_once('public/class-wdm-product-bundle-type-display.php');
            $GLOBALS['productDisplay'] = new WdmProductBundleTypeDisplay();

            $this->cpbThemeCompatibility();
            
            // Subscriptions
            if (\isSubscriptionActive()) {
                require_once('includes/subscriptions/class-wdm-cpb-subscription.php');
            }
        }

        /**
        * Loads the CPB Plugin's text-domain.
        */
        public function wdmCpbLoadTextDomain()
        {
            load_plugin_textdomain('brewpixel-wc-custom-product-bundles', false, dirname(plugin_basename(__FILE__)) . '/languages/');
        }

        /**
        * If the CPB Plugin is activated :
        * In the add-new product if selected product-type is CPB
        * Set the session variable for CPB Product.
        */
        public function wdmProductTypeCpb()
        {
            $act = $_POST['act'];
            $wdm_cpb = "wdm_cpb";

            if ($act == 'set') {
                $_SESSION[$wdm_cpb] = 1;
            } else {
                unset($_SESSION[$wdm_cpb]);
            }

            die();
        }

        public function createGlobalProductObject()
        {
            global $cpbProduct;

            if (!empty($cpbProduct)) {
                return;
            }
            if (is_product()) {
                $cpbProduct = wc_get_product(get_the_ID());
            }
        }

        /**
        * Gets the desktop layout from meta table selected for the CPB.
        * @param int $product_id CPB Product Id.
        * @return string Desktop template layout path selected for CPB.
        */
        public function getDesktopLayout($product_id)
        {
            $layout = get_post_meta($product_id, '_wdm_desktop_layout', true);

            // Set Vertical layout as a default layout
            if (empty($layout)) {
                $layout = $this->pluginPath() . '/templates/product-layouts/desktop-layouts/vertical';
            }

            return apply_filters('wdm_cpb_desktop_layout', $layout, $product_id);
        }
        /**
        * Functions to be performed when acttivating CPB.
        * Check if the CPB is active or not in options table.
        * If not, check if the product-type of CPB exists or not in the terms
        * and the taxonomies table.
        * If it does not exists , insert new custom product type, in the terms * and taxonomies table.
        * Activate the CPB Plugin.
        */
        public function wdmActivate()
        {
            global $wpdb;

            $is_active = get_option('woocommerce_product_bundles_active', false);

            if ($is_active == false) {
                $bundle_type_exists = false;

                $product_type_terms = get_terms('product_type', array('hide_empty' => false));

                foreach ($product_type_terms as $product_type_term) {
                    if ($product_type_term->slug === 'wdm_bundle_product') {
                        $bundle_type_exists = true;
                    }
                }

                // Check for existing 'bundle' slug and if it exists, modify it
                if ($existing_term_id = term_exists('wdm_bundle_product')) {
                    $wpdb->update(
                        $wpdb->terms,
                        array(
                            'slug' => 'wdm_bundle_product'
                        ),
                        array(
                            'term_id' => $existing_term_id
                        )
                    );
                }

                if (! $bundle_type_exists) {
                    wp_insert_term(
                        'wdm_bundle_product', // name
                        'product_type',
                        array(
                            'description'=> 'CPB',
                            'slug' => 'wdm_bundle_product',
                        )
                    );
                }

                add_option('woocommerce_product_bundles_active', true);
            }
        }
        /**
        * Gets the active theme stylesheet of the system.
        * Include the template which provide the active theme compatibility.
        */
        public function cpbThemeCompatibility()
        {
            $active_theme = wp_get_theme();
            $theme = wp_get_theme($active_theme->Template);
            $foundTheme = $theme->get_stylesheet();

            switch ($foundTheme) {
                case 'shopkeeper':
                    require_once('theme-compatibility/brewpixel-wc-custom-product-bundles-shopkeeper-addon.php');
                    break;
                case 'enfold':
                    require_once('theme-compatibility/brewpixel-wc-custom-product-bundles-enfold-addon.php');
                    break;
                case 'flatsome':
                    require_once('theme-compatibility/brewpixel-wc-custom-product-bundles-flatsome-addon.php');
                    break;
                case 'Avada':
                    require_once('theme-compatibility/brewpixel-wc-custom-product-bundles-avada-addon.php');
                    break;
                case 'theretailer':
                    require_once('theme-compatibility/brewpixel-wc-custom-product-bundles-theretailer-addon.php');
                    break;
                case 'depot':
                    require_once('theme-compatibility/brewpixel-wc-custom-product-bundles-depot-addon.php');
                    break;
                case 'Atelier':
                    require_once('theme-compatibility/brewpixel-wc-custom-product-bundles-atelier-addon.php');
                    break;
                case 'salient':
                    require_once('theme-compatibility/brewpixel-wc-custom-product-bundles-salient-addon.php');
                    break;
                default:
                    break;
            }
        }
    }
}
